/*
 * Copyright (c) 2015-2018, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
 *  ======== rfEasyLinkRx.c ========
 */
/* XDCtools Header files */
#include <xdc/std.h>
#include <xdc/runtime/Assert.h>
#include <xdc/runtime/Error.h>
#include <xdc/runtime/System.h>

/* BIOS Header files */
#include <ti/sysbios/BIOS.h>
#include <ti/sysbios/knl/Task.h>
#include <ti/sysbios/knl/Semaphore.h>
#include <ti/sysbios/knl/Clock.h>
#include <ti/drivers/UART.h>
#include <ti/drivers/GPIO.h>
#include "hw_types.h"
#include "hw_memmap.h"
#include "hw_aon_pmctl.h"
#include "sys_ctrl.h"
#include "rfEasyLinkRx.h"
//#include "PowerCC26X2.h"

#define CUSTOM_CRC_USED
#define TI_FW_VERSION       106

/* TI-RTOS Header files */
#include <ti/drivers/PIN.h>

/* Board Header files */
#include "ti_drivers_config.h"

/* EasyLink API Header files */
#include "easylink/EasyLink.h"

/***** Defines *****/

/* Undefine to remove async mode */
//#define RFEASYLINKRX_ASYNC

#define RFEASYLINKRX_TASK_STACK_SIZE    2048
#define RFEASYLINKRX_TASK_PRIORITY      1

#define UART_TASK_STACK_SIZE            2048
#define UART_TASK_PRIORITY              2

#define RFEASYLINK_NONE_MODE            0
#define RFEASYLINKRX_MODE               1
#define RFEASYLINKTX_MODE               2

#define RFEASYLINKECHO_PAYLOAD_LENGTH   30

#define CONCENTRATORRADIO_ACK_DELAY     120

#define _TPMS_SERIAL_NUMBER_LEN         16
#define  DMA_BUF_SIZE                   50

#define _TPMS_TIRE_POSITION_LEN         2

#define _TPMS_CAR_NUMBER                20

/* Defines */
#define MAX_MSG_LENGTH                  64

#define FEATURE_2ND_SENSOR
//#define FEATURE_SLEEP_MODE
//#define FEATURE_CORECHIPS

//Response (Get id)
/* 1 */
#define HANDLE(name) uint8_t name##Handler(char* out, TCmdTbl* tbl, char* args)
HANDLE(TxModeSwitch);
HANDLE(RxModeSwitch);
HANDLE(GetIdResponseCommand);
HANDLE(SetInfoResponseCommand);
HANDLE(SetCompleteResponseCommand);
HANDLE(DeleteInfoResponseCommand);
HANDLE(NewStockeResponseCommand);
HANDLE(RestockeResponseCommand);
HANDLE(DisposeResponseCommand);
HANDLE(SleepModeResponseCommand);

/* 2 */
typedef enum
{
    RF_TX_MODE,
    RF_RX_MODE,
    RESPONE_GET_ID_MODE,
    RESPONE_SET_INFO_MODE,
    RESPONE_SET_COMPLETE_MODE,
    RESPONE_DELETE_INFO_MODE,
    RESPONE_NEWSTOCKE_INFO_MODE,
    RESPONE_RESTOCKE_MODE,
    RESPONE_DISPOSE_MODE,
#ifdef FEATURE_SLEEP_MODE
    RESPONE_SLEEP_MODE,
#endif
} Tcmd;

/* 3 */
#define TBL(idx, cmd, m1, m2, hd) { idx, cmd, sizeof(cmd)-1, m1, m2, hd##Handler }
static const TCmdTbl parserCmds[] =
{
    TBL(RF_TX_MODE,                 "CMD001",    M_UNABLE, M_UNABLE, TxModeSwitch),
    TBL(RF_RX_MODE,                 "CMD002",    M_UNABLE, M_UNABLE, RxModeSwitch),
    TBL(RESPONE_GET_ID_MODE,        "KUMHO1101", M_UNABLE, M_UNABLE, GetIdResponseCommand),
    TBL(RESPONE_SET_INFO_MODE,      "KUMHO1102", M_UNABLE, M_UNABLE, SetInfoResponseCommand),
    TBL(RESPONE_SET_COMPLETE_MODE,  "KUMHO1103", M_UNABLE, M_UNABLE, SetCompleteResponseCommand),
    TBL(RESPONE_DELETE_INFO_MODE,   "KUMHO1104", M_UNABLE, M_UNABLE, DeleteInfoResponseCommand),
    TBL(RESPONE_NEWSTOCKE_INFO_MODE,"KUMHO1201", M_UNABLE, M_UNABLE, NewStockeResponseCommand),
    TBL(RESPONE_RESTOCKE_MODE,      "KUMHO1202", M_UNABLE, M_UNABLE, RestockeResponseCommand),
    TBL(RESPONE_DISPOSE_MODE,       "KUMHO1203", M_UNABLE, M_UNABLE, DisposeResponseCommand),
#ifdef FEATURE_SLEEP_MODE
    TBL(RESPONE_SLEEP_MODE,         "SLEEPMODE", M_UNABLE, M_UNABLE, SleepModeResponseCommand),
#endif
};

//static uint16_t seqNumber;

EasyLink_TxPacket txPacket = {{0}, 0, 0, {0}};

/* Message buffers */
static uint8_t msg[MAX_MSG_LENGTH];

/* Pin driver handle */
static PIN_Handle ledPinHandle;
//static PIN_State ledPinState;

/***** Variable declarations *****/
static Task_Params rxTaskParams;

Task_Struct rxTask;    /* not static so you can see in ROV */
Task_Struct uartTask;

static uint8_t rxTaskStack[RFEASYLINKRX_TASK_STACK_SIZE];
static uint8_t uartTaskStack[RFEASYLINKRX_TASK_STACK_SIZE];


/* The RX Output struct contains statistics about the RX operation of the radio */
PIN_Handle pinHandle;

#ifdef RFEASYLINKRX_ASYNC
static Semaphore_Handle rxDoneSem;
#endif

UART_Handle uart;
static UART_Params uartParams;
static EasyLink_RxPacket rxPacketDump;
static EasyLink_Params easyLink_params;

PIN_Handle hPins;
PIN_State   PinState;


uint8_t switchRxTx = RFEASYLINKRX_MODE;
uint8_t is_mode_control = 0;
uint8_t g_tx_trigger;

unsigned char   hex8[64] = {0x00,};
uint8_t MANCHESTER_ZERO = (0x2);
uint8_t MANCHESTER_ONE = (0x1);



uint8_t crctable[256] = {
    0x00, 0x07, 0x0E, 0x09, 0x1C, 0x1B, 0x12, 0x15,
    0x38, 0x3F, 0x36, 0x31, 0x24, 0x23, 0x2A, 0x2D,
    0x70, 0x77, 0x7E, 0x79, 0x6C, 0x6B, 0x62, 0x65,
    0x48, 0x4F, 0x46, 0x41, 0x54, 0x53, 0x5A, 0x5D,
    0xE0, 0xE7, 0xEE, 0xE9, 0xFC, 0xFB, 0xF2, 0xF5,
    0xD8, 0xDF, 0xD6, 0xD1, 0xC4, 0xC3, 0xCA, 0xCD,
    0x90, 0x97, 0x9E, 0x99, 0x8C, 0x8B, 0x82, 0x85,
    0xA8, 0xAF, 0xA6, 0xA1, 0xB4, 0xB3, 0xBA, 0xBD,
    0xC7, 0xC0, 0xC9, 0xCE, 0xDB, 0xDC, 0xD5, 0xD2,
    0xFF, 0xF8, 0xF1, 0xF6, 0xE3, 0xE4, 0xED, 0xEA,
    0xB7, 0xB0, 0xB9, 0xBE, 0xAB, 0xAC, 0xA5, 0xA2,
    0x8F, 0x88, 0x81, 0x86, 0x93, 0x94, 0x9D, 0x9A,
    0x27, 0x20, 0x29, 0x2E, 0x3B, 0x3C, 0x35, 0x32,
    0x1F, 0x18, 0x11, 0x16, 0x03, 0x04, 0x0D, 0x0A,
    0x57, 0x50, 0x59, 0x5E, 0x4B, 0x4C, 0x45, 0x42,
    0x6F, 0x68, 0x61, 0x66, 0x73, 0x74, 0x7D, 0x7A,
    0x89, 0x8E, 0x87, 0x80, 0x95, 0x92, 0x9B, 0x9C,
    0xB1, 0xB6, 0xBF, 0xB8, 0xAD, 0xAA, 0xA3, 0xA4,
    0xF9, 0xFE, 0xF7, 0xF0, 0xE5, 0xE2, 0xEB, 0xEC,
    0xC1, 0xC6, 0xCF, 0xC8, 0xDD, 0xDA, 0xD3, 0xD4,
    0x69, 0x6E, 0x67, 0x60, 0x75, 0x72, 0x7B, 0x7C,
    0x51, 0x56, 0x5F, 0x58, 0x4D, 0x4A, 0x43, 0x44,
    0x19, 0x1E, 0x17, 0x10, 0x05, 0x02, 0x0B, 0x0C,
    0x21, 0x26, 0x2F, 0x28, 0x3D, 0x3A, 0x33, 0x34,
    0x4E, 0x49, 0x40, 0x47, 0x52, 0x55, 0x5C, 0x5B,
    0x76, 0x71, 0x78, 0x7F, 0x6A, 0x6D, 0x64, 0x63,
    0x3E, 0x39, 0x30, 0x37, 0x22, 0x25, 0x2C, 0x2B,
    0x06, 0x01, 0x08, 0x0F, 0x1A, 0x1D, 0x14, 0x13,
    0xAE, 0xA9, 0xA0, 0xA7, 0xB2, 0xB5, 0xBC, 0xBB,
    0x96, 0x91, 0x98, 0x9F, 0x8A, 0x8D, 0x84, 0x83,
    0xDE, 0xD9, 0xD0, 0xD7, 0xC2, 0xC5, 0xCC, 0xCB,
    0xE6, 0xE1, 0xE8, 0xEF, 0xFA, 0xFD, 0xF4, 0xF3
};


/*************************************
 * Added for MKW01 CRC
 *************************************/
uint16_t ComputeCrc_MKW01(uint16_t crc, uint8_t dataByte, uint16_t polynomial) {
    uint8_t i;
    for (i = 0; i < 8; i++) {
        if ((((crc & 0x8000) >> 8) ^ (dataByte & 0x80)) != 0) {
            crc <<= 1; // shift left once
            crc ^= polynomial; // XOR with polynomial
        } else {
            crc <<= 1; // shift left once
        }
        dataByte <<= 1; // Next data bit
    }
    return crc;
}

uint16_t Calculate_CRC_MKW01(uint8_t *buffer, uint8_t length) {
    uint8_t i = 0;
    uint16_t crc = 0;
    uint16_t polynomial = 0;
    polynomial = 0x1021;
    crc = 0x1D0F;
    for (i = 0; i < length; i++) {
        crc = ComputeCrc_MKW01(crc, buffer[i], polynomial);
    }
    return ((uint16_t) (~crc));
}


uint8_t CRC8_Update( uint8_t *data, uint16_t length )
{
	uint8_t crc = 0;

	while(length--)
	{
	    crc = crctable[crc ^ *data++];
	}

	return crc;
}


uint16_t ManchesterCode_EncodeChar(uint8_t byte)
{
  uint16_t encoded_data= 0x00;
  int      bitIdx = 0;

  for (bitIdx = 7; bitIdx >= 0; bitIdx--) {
    encoded_data <<= 2;
    if (((byte >> bitIdx) & 0x1) == 0x1) {
      encoded_data |= MANCHESTER_ONE;
    } else {
      encoded_data |= MANCHESTER_ZERO;
    }
  }

  return encoded_data;
}


uint8_t ManchesterCode_DecodeChar(uint16_t encoded_data)
{
  uint8_t byte = 0x0;
  int     bitIdx = 0;

  for (bitIdx = 7; bitIdx >= 0; bitIdx--)
  {
    byte <<= 1;
    if (((encoded_data >> (bitIdx << 1)) & 0x3) == MANCHESTER_ONE) {
      byte |= 0x01;
    }
  }

  return byte;
}


/* return: encoded string length */
uint8_t ManchesterCode_EncodeString(uint8_t* pDecData,
                                    uint16_t* pEncoded_data, uint8_t decoded_len)
{
  uint8_t len = 0, i;

  for (i = 0; i < decoded_len; ++i)
  {
    pEncoded_data[i] = ManchesterCode_EncodeChar(pDecData[i]);
  }
  len += (decoded_len * (sizeof(uint8_t) * 2));

  return len;
}


/* return: decoded string length */
size_t ManchesterCode_DecodeString(uint16_t* pData,
                                    uint8_t* pEncoded_data, size_t encoded_len)
{
  uint8_t decoded_len, i;

  decoded_len = (encoded_len - (sizeof(uint8_t) * 2));

  for (i = 0; i < decoded_len; ++i) {
    pEncoded_data[i] = ManchesterCode_DecodeChar(pData[i]);
  }
  return decoded_len;
}


void Tire_Command_Response_State(uint8_t *buffer, uint8_t result)
{
    uint8_t i;

    for(i=0; i < 10; i++)
    {
        rxPacketDump.payload[i] = buffer[i];
    }

    rxPacketDump.payload[i++] = result;
    TxCommandHandler(rxPacketDump.payload, i);
}


//Transmits the data received from Tire setting machine.
void Tire_Get_id_RequestRxData(void)
{
    uint8_t tmpbuf[52];
    uint8_t index = 0;
    uint8_t i, k;
    uint8_t hex_word[16] = {'0','1','2','3','4','5','6','7','8','9','A','B','C','D','E','F'};

    memset(&tmpbuf, 0, sizeof(tmpbuf));

    tmpbuf[index++]  = 'K';
    tmpbuf[index++]  = '1';
    tmpbuf[index++]  = '1';
    tmpbuf[index++]  = '0';
    tmpbuf[index++]  = '1';

    k = 2;
    for (i=0; i< 8; i++) //8byte  -> serial Number
    {
        tmpbuf[index++] = hex_word[(rxPacketDump.payload[k] >> 4) &0x0f];
        tmpbuf[index++] = hex_word[(rxPacketDump.payload[k] & 0x0f)];
        k++;
    }

    for (i=index; i< DMA_BUF_SIZE; i++)
    {
        tmpbuf[index++] = 0x0a;
    }

    GPIO_write(CONFIG_GPIO_RTS, 1);
    UART_write(uart, tmpbuf, index);
    GPIO_write(CONFIG_GPIO_RTS, 0);
}


void Tire_Set_InfoRxData(void)
{
    uint8_t i, k;
    uint8_t tmpbuf[64] = {0x00,};
    uint8_t index = 0;
    uint8_t hex_word[16] = {'0','1','2','3','4','5','6','7','8','9','A','B','C','D','E','F'};
    uint8_t rx_len = 30;
    uint16_t verification_Value;
    uint16_t verification_rx;

    memset(&tmpbuf, 0, sizeof(tmpbuf));

    tmpbuf[index++]  = 'K';
    tmpbuf[index++]  = '1';
    tmpbuf[index++]  = '1';
    tmpbuf[index++]  = '0';
    tmpbuf[index++]  = '2';

    k = 2;
    for (i=0; i< 8; i++) //8byte  -> serial Number
    {
        tmpbuf[index++] = hex_word[(rxPacketDump.payload[k] >> 4) &0x0f];
        tmpbuf[index++] = hex_word[(rxPacketDump.payload[k] & 0x0f)];
        k++;
    }

    k = 10; // Tire Position
    tmpbuf[index++] = hex_word[(rxPacketDump.payload[k] >> 4) &0x0f];
    tmpbuf[index++] = hex_word[(rxPacketDump.payload[k] & 0x0f)];

    k = 11 + 6; // Tire RFID + 6Byte (fixed value)
    for (i=0; i< 6; i++)
    {
        tmpbuf[index++] = hex_word[(rxPacketDump.payload[k] >> 4) &0x0f];
        tmpbuf[index++] = hex_word[(rxPacketDump.payload[k] & 0x0f)];
        k++;
    }

    k = 23; //Tire Sensor Type
    tmpbuf[index++] = hex_word[(rxPacketDump.payload[k] >> 4) &0x0f];
    tmpbuf[index++] = hex_word[(rxPacketDump.payload[k] & 0x0f)];

    k = 24; //Tire Sensor ID
    for (i=0; i< 4; i++)
    {
        tmpbuf[index++] = hex_word[(rxPacketDump.payload[k] >> 4) &0x0f];
        tmpbuf[index++] = hex_word[(rxPacketDump.payload[k] & 0x0f)];
        k++;
    }

    verification_rx = rxPacketDump.payload[rx_len-2];
    verification_rx = (verification_rx << 8) | rxPacketDump.payload[rx_len-1];
    verification_Value = Calculate_CRC_MKW01( &rxPacketDump.payload[0], (uint8_t)(rx_len-2));


    if(verification_rx != verification_Value) {
        Tire_Command_Response_State(rxPacketDump.payload, 0x01);
        return;
    }

    for (i=index; i< DMA_BUF_SIZE; i++)
    {
        tmpbuf[index++] = 0x0a;
    }

    GPIO_write(CONFIG_GPIO_RTS, 1);
    UART_write(uart, tmpbuf, index);
    GPIO_write(CONFIG_GPIO_RTS, 0);
}


void Tire_Set_Complete(void)
{
    uint8_t tmpbuf[64];
    uint8_t index = 0, i, k;
    uint8_t rx_len = 12;
    uint8_t hex_word[16] = {'0','1','2','3','4','5','6','7','8','9','A','B','C','D','E','F'};
    uint16_t verification_Value;
    uint16_t verification_rx;

    memset(&tmpbuf, 0, sizeof(tmpbuf));

    tmpbuf[index++]  = 'K';
    tmpbuf[index++]  = '1';
    tmpbuf[index++]  = '1';
    tmpbuf[index++]  = '0';
    tmpbuf[index++]  = '3';

    k = 2;
    for (i=0; i< 8; i++) //8byte  -> serial Number
    {
        tmpbuf[index++] = hex_word[(rxPacketDump.payload[k] >> 4) &0x0f];
        tmpbuf[index++] = hex_word[(rxPacketDump.payload[k] & 0x0f)];
        k++;
    }

    verification_rx = rxPacketDump.payload[rx_len-2];
    verification_rx = (verification_rx << 8) | rxPacketDump.payload[rx_len-1];
    verification_Value = Calculate_CRC_MKW01( &rxPacketDump.payload[0], (uint8_t)(rx_len-2));

    if(verification_rx != verification_Value) {
        Tire_Command_Response_State(rxPacketDump.payload, 0x01);
        return;
    }

    for (i=index; i< DMA_BUF_SIZE; i++)
    {
        tmpbuf[index++] = 0x0a;
    }

    GPIO_write(CONFIG_GPIO_RTS, 1);
    UART_write(uart, tmpbuf, index);
    GPIO_write(CONFIG_GPIO_RTS, 0);
}


void Tire_Delete_Tire_Info(void)
{
    uint8_t i, k;
    uint8_t rx_len = 13;
    uint8_t tmpbuf[64] = {0x00,};
    uint8_t index = 0;
    uint8_t hex_word[16] = {'0','1','2','3','4','5','6','7','8','9','A','B','C','D','E','F'};
    uint16_t verification_Value;
    uint16_t verification_rx;

    memset(&tmpbuf, 0, sizeof(tmpbuf));

    tmpbuf[index++]  = 'K';
    tmpbuf[index++]  = 'O';
    tmpbuf[index++]  = '1';
    tmpbuf[index++]  = '1';
    tmpbuf[index++]  = '0';
    tmpbuf[index++]  = '4';

    k = 2;
    for (i=0; i< 9; i++)
    {
        tmpbuf[index++] = hex_word[(rxPacketDump.payload[k] >> 4) &0x0f];
        tmpbuf[index++] = hex_word[(rxPacketDump.payload[k] & 0x0f)];
        k++;
    }

    verification_rx = rxPacketDump.payload[rx_len-2];
    verification_rx = (verification_rx << 8) | rxPacketDump.payload[rx_len-1];
    verification_Value = Calculate_CRC_MKW01( &rxPacketDump.payload[0], (uint8_t)(rx_len-2));


    if(verification_rx != verification_Value) {
        Tire_Command_Response_State(rxPacketDump.payload, 0x01);
        return;
    }

    for (i=index; i< DMA_BUF_SIZE; i++)
    {
        tmpbuf[index++] = 0x0a;
    }

    GPIO_write(CONFIG_GPIO_RTS, 1);
    UART_write(uart, tmpbuf, index);
    GPIO_write(CONFIG_GPIO_RTS, 0);
}

void Tire_NewStocke_Info(void)
{
    uint8_t i, k;
    uint8_t rx_len = 28;
    uint8_t tmpbuf[64] = {0x00,};
    uint8_t index = 0;
    uint8_t hex_word[16] = {'0','1','2','3','4','5','6','7','8','9','A','B','C','D','E','F'};
    uint16_t verification_Value;
    uint16_t verification_rx;

    memset(&tmpbuf, 0, sizeof(tmpbuf));

    tmpbuf[index++]  = 'K';
    tmpbuf[index++]  = '1';
    tmpbuf[index++]  = '2';
    tmpbuf[index++]  = '0';
    tmpbuf[index++]  = '1';

    k = 2;
    for (i=0; i< 8; i++) //8byte -> serial Number
    {
        tmpbuf[index++] = hex_word[(rxPacketDump.payload[k] >> 4) &0x0f];
        tmpbuf[index++] = hex_word[(rxPacketDump.payload[k] & 0x0f)];
        k++;
    }

    k = 10 + 6; //Tire RFID + 6Byte (fixed value)
    for (i=0; i< 6; i++)
    {
        tmpbuf[index++] = hex_word[(rxPacketDump.payload[k] >> 4) &0x0f];
        tmpbuf[index++] = hex_word[(rxPacketDump.payload[k] & 0x0f)];
        k++;
    }

    k = 22; //Tire Sensor ID
    for (i=0; i< 4; i++)
    {
        tmpbuf[index++] = hex_word[(rxPacketDump.payload[k] >> 4) &0x0f];
        tmpbuf[index++] = hex_word[(rxPacketDump.payload[k] & 0x0f)];
        k++;
    }

    verification_rx = rxPacketDump.payload[rx_len-2];
    verification_rx = (verification_rx << 8) | rxPacketDump.payload[rx_len-1];
    verification_Value = Calculate_CRC_MKW01( &rxPacketDump.payload[0], (uint8_t)(rx_len-2));


    if(verification_rx != verification_Value) {
        Tire_Command_Response_State(rxPacketDump.payload, 0x01);
        return;
    }

    for (i=index; i< DMA_BUF_SIZE; i++)
    {
        tmpbuf[index++] = 0x0a;
    }

    GPIO_write(CONFIG_GPIO_RTS, 1);
    UART_write(uart, tmpbuf, index);
    GPIO_write(CONFIG_GPIO_RTS, 0);
}


void Tire_Restocke_Info(void)
{
    uint8_t i, k;
    uint8_t tmpbuf[64] = {0x00,};
    uint8_t rx_len = 28;
    uint8_t index = 0;
    uint8_t hex_word[16] = {'0','1','2','3','4','5','6','7','8','9','A','B','C','D','E','F'};
    uint16_t verification_Value;
    uint16_t verification_rx;

    memset(&tmpbuf, 0, sizeof(tmpbuf));

    tmpbuf[index++]  = 'K';
    tmpbuf[index++]  = '1';
    tmpbuf[index++]  = '2';
    tmpbuf[index++]  = '0';
    tmpbuf[index++]  = '2';

    k = 2;
    for (i=0; i< 8; i++) //8byte  -> serial Number
    {
        tmpbuf[index++] = hex_word[(rxPacketDump.payload[k] >> 4) &0x0f];
        tmpbuf[index++] = hex_word[(rxPacketDump.payload[k] & 0x0f)];
        k++;
    }

    k = 10 + 6; // Tire RFID + 6Byte (fixed value)
    for (i=0; i< 6; i++)
    {
        tmpbuf[index++] = hex_word[(rxPacketDump.payload[k] >> 4) &0x0f];
        tmpbuf[index++] = hex_word[(rxPacketDump.payload[k] & 0x0f)];
        k++;
    }

    k = 22;
    for (i=0; i< 4; i++)
    {
        tmpbuf[index++] = hex_word[(rxPacketDump.payload[k] >> 4) &0x0f];
        tmpbuf[index++] = hex_word[(rxPacketDump.payload[k] & 0x0f)];
        k++;
    }

    verification_rx = rxPacketDump.payload[rx_len-2];
    verification_rx = (verification_rx << 8) | rxPacketDump.payload[rx_len-1];
    verification_Value = Calculate_CRC_MKW01( &rxPacketDump.payload[0], (uint8_t)(rx_len-2));


    if(verification_rx != verification_Value) {
        Tire_Command_Response_State(rxPacketDump.payload, 0x01);
        return;
    }

    for (i=index; i< DMA_BUF_SIZE; i++)
    {
        tmpbuf[index++] = 0x0a;
    }

    GPIO_write(CONFIG_GPIO_RTS, 1);
    UART_write(uart, tmpbuf, index);
    GPIO_write(CONFIG_GPIO_RTS, 0);
}


void Tire_Dispose_Info(void)
{
    uint8_t i, k;
    uint8_t rx_len = 28;
    uint8_t tmpbuf[64] = {0x00,};
    uint8_t index = 0;
    uint8_t hex_word[16] = {'0','1','2','3','4','5','6','7','8','9','A','B','C','D','E','F'};
    uint16_t verification_Value;
    uint16_t verification_rx;

    memset(&tmpbuf, 0, sizeof(tmpbuf));

    tmpbuf[index++]  = 'K';
    tmpbuf[index++]  = '1';
    tmpbuf[index++]  = '2';
    tmpbuf[index++]  = '0';
    tmpbuf[index++]  = '3';

    k = 2;
    for (i=0; i< 8; i++) //8byte  -> serial Number
    {
        tmpbuf[index++] = hex_word[(rxPacketDump.payload[k] >> 4) &0x0f];
        tmpbuf[index++] = hex_word[(rxPacketDump.payload[k] & 0x0f)];
        k++;
    }

    k = 10 + 6; // Tire RFID + 6Byte (fixed value)
    for (i=0; i< 6; i++)
    {
        tmpbuf[index++] = hex_word[(rxPacketDump.payload[k] >> 4) &0x0f];
        tmpbuf[index++] = hex_word[(rxPacketDump.payload[k] & 0x0f)];
        k++;
    }

    k = 22;
    for (i=0; i< 4; i++)
    {
        tmpbuf[index++] = hex_word[(rxPacketDump.payload[k] >> 4) &0x0f];
        tmpbuf[index++] = hex_word[(rxPacketDump.payload[k] & 0x0f)];
        k++;
    }

    verification_rx = rxPacketDump.payload[rx_len-2];
    verification_rx = (verification_rx << 8) | rxPacketDump.payload[rx_len-1];
    verification_Value = Calculate_CRC_MKW01( &rxPacketDump.payload[0], (uint8_t)(rx_len-2));

    if(verification_rx != verification_Value) {
        Tire_Command_Response_State(rxPacketDump.payload, 0x01);
        return;
    }

    for (i=index; i< DMA_BUF_SIZE; i++)
    {
        tmpbuf[index++] = 0x0a;
    }

    GPIO_write(CONFIG_GPIO_RTS, 1);
    UART_write(uart, tmpbuf, index);
    GPIO_write(CONFIG_GPIO_RTS, 0);
}

void customDisplayRxData(void)
{
    uint8_t hex_word[16] = {'0','1','2','3','4','5','6','7','8','9','A','B','C','D','E','F'};

    uint8_t tmpbuf[256];
    uint8_t index = 0;
    uint8_t RF_CrcNotOk = 0;
    uint8_t i;
#if (TI_FW_VERSION < 106)
    static uint16_t counter = 0;
#endif
    uint16_t verification_Value;
    uint16_t verification_rx;
#if (TI_FW_VERSION < 106)
    uint8_t rx_len = 20;
#else
#ifdef FEATURE_2ND_SENSOR
    uint8_t rx_len = rxPacketDump.len;
#else
    uint8_t rx_len = 18;
#endif
#endif

    memset(&tmpbuf, 0, sizeof(tmpbuf));
#if 0
    tmpbuf[0]  = '[';
    tmpbuf[1]  = 'K';
    tmpbuf[2]  = 'U';
    tmpbuf[3]  = 'M';
    tmpbuf[4]  = 'H';
    tmpbuf[5]  = 'O';
    tmpbuf[6]  = '1';
    tmpbuf[7]  = '1';
    tmpbuf[8]  = '0';
    tmpbuf[9]  = '0';

    index = 10;
#endif

    for (i=0; i<rx_len; i++) // Do not display constant bytes at the end
    {
        tmpbuf[index++] = hex_word[(rxPacketDump.payload[i] >> 4) &0x0f];
        tmpbuf[index++] = hex_word[(rxPacketDump.payload[i] & 0x0f)];
    }

#if 0
#if (TI_FW_VERSION < 106)
        //Status Acquisition for Comp functions --> TI_FW_VERSION
    tmpbuf[rx_len*2] = hex_word[(TI_FW_VERSION >> 4) &0x0f];
    tmpbuf[rx_len*2+1] = hex_word[(TI_FW_VERSION & 0x0f)];
#else
    tmpbuf[index++] = hex_word[(TI_FW_VERSION >> 4) &0x0f];
    tmpbuf[index++] = hex_word[(TI_FW_VERSION & 0x0f)];
#endif

#ifdef CUSTOM_CRC_USED
# if (TI_FW_VERSION < 106)
    verification_rx = rxPacketDump.payload[rx_len-1];
    verification_rx = (verification_rx << 8) | rxPacketDump.payload[rx_len];
    verification_Value = Calculate_CRC_MKW01( &rxPacketDump.payload[0], (uint8_t)(rx_len-1));
# else
    verification_rx = rxPacketDump.payload[rx_len-2];
    verification_rx = (verification_rx << 8) | rxPacketDump.payload[rx_len-1];
    verification_Value = Calculate_CRC_MKW01( &rxPacketDump.payload[0], (uint8_t)(rx_len-2));
# endif

    if(verification_rx != verification_Value) {
        RF_CrcNotOk = 1;
    }
#else
    if(rxPacketDump.payload[rx_len-2] != 0xC0 || rxPacketDump.payload[rx_len-1] != 0xC1 || rxPacketDump.payload[rx_len-0] != 0xC2) {
        RF_CrcNotOk = 1;
    }
#endif

# if (TI_FW_VERSION < 106)
    tmpbuf[index++] = hex_word[((uint8_t)(counter>>8) >> 4)&0x0f]; // MKW01 RF counter
    tmpbuf[index++] = hex_word[((uint8_t)(counter>>8) & 0x0f)];
    tmpbuf[index++] = hex_word[((uint8_t)(counter&0x00FF) >> 4)&0x0f];
    tmpbuf[index++] = hex_word[((uint8_t)(counter&0x00FF) & 0x0f)];
    counter++;
#endif
    tmpbuf[index++] = hex_word[((uint8_t)(RF_CrcNotOk) >> 4)&0x0f]; // 0 if CRC OK, 1 if not OK
    tmpbuf[index++] = hex_word[((uint8_t)(RF_CrcNotOk) & 0x0f)];
    tmpbuf[index++] = hex_word[((uint8_t)(rxPacketDump.rssi) >> 4)&0x0f]; // RSSI value
    tmpbuf[index++] = hex_word[((uint8_t)(rxPacketDump.rssi) & 0x0f)];
    tmpbuf[index++] = ']';
    tmpbuf[index++] = 0x0a;

#ifndef FEATURE_2ND_SENSOR
    for (i=index; i< DMA_BUF_SIZE; i++)
    {
        tmpbuf[index++] = 0x0a;
    }
#endif
#endif
    tmpbuf[index++] = '\r';
    tmpbuf[index++] = '\n';

    GPIO_write(CONFIG_GPIO_RTS, 1);
    UART_write(uart, tmpbuf, index);
    GPIO_write(CONFIG_GPIO_RTS, 0);
}


/*****************************************************************************
* Function Name: rtStrAtol()
******************************************************************************
* Summary:
*
* Parameters:
* None
*
* Return:
*
* Theory:
*
* Side Effects:
*
* Note:
* None
*****************************************************************************/
uint64_t rtStrAtol(const uint8_t *str)
{
    uint8_t  m = 0;
    uint64_t r = 0;

    while ( *str && ( *str < '0' || *str > '9' ) )
    {
        if ( *str == '-' )
        {
            m = 1 - m;
        }
        else if ( *str > ' ' && *str != '+' )
        {
            return 0;
        }

        str++;
    }

    while ( '0' <= *str && *str <= '9' )
    {
        r = r*10 + *str++ - '0';
    }

    return m ? -r : r;
}


/*****************************************************************************
* Function Name: rtStrAtoi()
******************************************************************************
* Summary:
*
* Parameters:
* None
*
* Return:
*
* Theory:
*
* Side Effects:
*
* Note:
* None
*****************************************************************************/
uint64_t rtStrAtoi(const uint8_t *str)
{
    return (uint64_t)rtStrAtol( str );
}


/*****************************************************************************
* Function Name: rtint64tohex()
******************************************************************************
* Summary:
*
* Parameters:
* None
*
* Return:
*
* Theory:
*
* Side Effects:
*
* Note:
* None
*****************************************************************************/
uint64_t rtint64tohex(uint64_t digit)
{
    uint64_t mod = 0;
    uint64_t hex = 0;
    uint64_t shift = 0;
    uint64_t count = 0;

    if( digit == 0 )
        return 0;

    do
    {
        mod = digit % 16;
        digit /= 16;

        if( digit > 16 )
        {
            shift = mod << (count * 4);
            hex |= shift;
        }
        else
        {
            hex |= digit << ((count * 4) + 4);
            hex |= (mod << (count * 4));
            break;
        }

        count++;
    } while(1);

    return hex;
}


/*=========================================================================
  FUNCTION:  rthex64to8
===========================================================================*/
/*!
    @brief

    @return

*/
/*=========================================================================*/
unsigned char* rthex64to8(uint64_t digit)
{
    unsigned char count =0;
    unsigned char hexval = 0;
    unsigned char *hexReturn;

    memset(hex8,0x00,sizeof(hex8));

    hexReturn = &hex8[0];

    if( digit == 0 )
        return 0;

    hexval = digit >> 48;
    hexReturn[count++] = hexval;

    hexval = digit >> 40;
    hexReturn[count++] = hexval;

    hexval = digit >> 32;
    hexReturn[count++] = hexval;

    hexval = digit >> 24;
    hexReturn[count++] = hexval;

    hexval = digit >> 16;
    hexReturn[count++] = hexval;

    hexval = digit >> 8;
    hexReturn[count++] = hexval;

    hexval = 0x000000000000FF & digit;
    hexReturn[count++] = hexval;

    return hexReturn;
}


/*****************************************************************************
* Function Name: rfRxTestmode
******************************************************************************
* Summary:
*
* Parameters:
* None
*
* Return:
*
* Theory:
*
* Side Effects:
*
* Note:
* None
*****************************************************************************/
void rfRxTestmode(void)
{
    uint8_t hex_word[16] = {'0','1','2','3','4','5','6','7','8','9','A','B','C','D','E','F'};

    uint8_t tmpbuf[256];
    uint8_t index = 0;
    uint8_t i;
    uint8_t rx_len = 18;

    memset(&tmpbuf, 0, sizeof(tmpbuf));

    tmpbuf[0]  = '[';

    index = 1;
    for (i=0; i<rx_len-2; i++) // Do not display constant bytes at the end
    {
        tmpbuf[index++] = hex_word[(rxPacketDump.payload[i] >> 4) &0x0f];
        tmpbuf[index++] = hex_word[(rxPacketDump.payload[i] & 0x0f)];
    }

    tmpbuf[index++] = ']';
    tmpbuf[index++] = 0x0a;

    GPIO_write(CONFIG_GPIO_RTS, 1);
    UART_write(uart, tmpbuf, index);
    GPIO_write(CONFIG_GPIO_RTS, 0);
}


/*****************************************************************************
* Function Name: TxCommandHandler
******************************************************************************
* Summary:
*
* Parameters:
* None
*
* Return:
*
* Theory:
*
* Side Effects:
*
* Note:
* None
*****************************************************************************/
static uint8_t TxCommandHandler(uint8_t* data, uint8_t len)
{
    uint16_t verification_Value;

    txPacket.len = len;
    memcpy(txPacket.payload, data, txPacket.len);

    verification_Value = Calculate_CRC_MKW01(txPacket.payload, txPacket.len);

    txPacket.payload[txPacket.len++] = verification_Value >> 8;
    txPacket.payload[txPacket.len++] = verification_Value & 0xFF;
    g_tx_trigger = 1;

    switchRxTx = RFEASYLINKTX_MODE;
    return TRUE;
}


/*****************************************************************************
* Function Name: rfRxCommandData
******************************************************************************
* Summary:
*  Classified_According to each CMD
* Parameters:
* None
*
* Return:
*
* Theory:
*
* Side Effects:
*
* Note:
* None
*****************************************************************************/
void rfRxCommandData(void)
{
    uint16_t rx_cmd;
    uint16_t verification_Value;
    uint16_t verification_rx;
    uint8_t  rx_len;
    //uint8_t  RF_CrcNotOk = 0;

    rx_cmd = (rxPacketDump.payload[0] << 8);
    rx_cmd |= rxPacketDump.payload[1];

    if(rx_cmd == COMMAND_GET_ID)
    {
        rx_len = 12;
        verification_rx = rxPacketDump.payload[rx_len-2];
        verification_rx = (verification_rx << 8) | rxPacketDump.payload[rx_len-1];
        verification_Value = Calculate_CRC_MKW01( &rxPacketDump.payload[0], (uint8_t)(rx_len-2));

        if(verification_rx != verification_Value) {
            //RF_CrcNotOk = 1;
        }
        else {
            Tire_Get_id_RequestRxData();
        }
    }
    else if(rx_cmd == COMMAND_SET_INFO)
    {
        rx_len = 30;
        verification_rx = rxPacketDump.payload[rx_len-2];
        verification_rx = (verification_rx << 8) | rxPacketDump.payload[rx_len-1];
        verification_Value = Calculate_CRC_MKW01( &rxPacketDump.payload[0], (uint8_t)(rx_len-2));

        if(verification_rx != verification_Value) {
            //RF_CrcNotOk = 1;
        }
        else {
            Tire_Set_InfoRxData();
        }
    }
    else if(rx_cmd == COMMAND_SET_COMPLETE)
    {
        rx_len = 12;
        verification_rx = rxPacketDump.payload[rx_len-2];
        verification_rx = (verification_rx << 8) | rxPacketDump.payload[rx_len-1];
        verification_Value = Calculate_CRC_MKW01( &rxPacketDump.payload[0], (uint8_t)(rx_len-2));

        if(verification_rx != verification_Value) {
            //RF_CrcNotOk = 1;
        }
        else {
            Tire_Set_Complete();
        }
    }
    else if(rx_cmd == COMMAND_DELETE_INFO)
    {
        rx_len = 13;
        verification_rx = rxPacketDump.payload[rx_len-2];
        verification_rx = (verification_rx << 8) | rxPacketDump.payload[rx_len-1];
        verification_Value = Calculate_CRC_MKW01( &rxPacketDump.payload[0], (uint8_t)(rx_len-2));

        if(verification_rx != verification_Value) {
            //RF_CrcNotOk = 1;
        }
        else {
            Tire_Delete_Tire_Info();
        }
    }
    else if(rx_cmd == COMMAND_TIRE_NEWSTOCKE)
    {
        rx_len = 28;
        verification_rx = rxPacketDump.payload[rx_len-2];
        verification_rx = (verification_rx << 8) | rxPacketDump.payload[rx_len-1];
        verification_Value = Calculate_CRC_MKW01( &rxPacketDump.payload[0], (uint8_t)(rx_len-2));

        if(verification_rx != verification_Value) {
            //RF_CrcNotOk = 1;
        }
        else {
            Tire_NewStocke_Info();
        }
    }
    else if(rx_cmd == COMMAND_TIRE_RESTOCKE)
    {
        rx_len = 28;
        verification_rx = rxPacketDump.payload[rx_len-2];
        verification_rx = (verification_rx << 8) | rxPacketDump.payload[rx_len-1];
        verification_Value = Calculate_CRC_MKW01( &rxPacketDump.payload[0], (uint8_t)(rx_len-2));

        if(verification_rx != verification_Value) {
            //RF_CrcNotOk = 1;
        }
        else {
            Tire_Restocke_Info();
        }
    }
    else if(rx_cmd == COMMAND_TIRE_DISPOSE)
    {
        rx_len = 28;
        verification_rx = rxPacketDump.payload[rx_len-2];
        verification_rx = (verification_rx << 8) | rxPacketDump.payload[rx_len-1];
        verification_Value = Calculate_CRC_MKW01( &rxPacketDump.payload[0], (uint8_t)(rx_len-2));

        if(verification_rx != verification_Value) {
            //RF_CrcNotOk = 1;
        }
        else {
            Tire_Dispose_Info();
        }
    }
    else
    {
        customDisplayRxData();
    }
}


/*****************************************************************************
* Function Name: TxModeSwitchHandler()
******************************************************************************
* Summary:
*
* Parameters:
* None
*
* Return:
*
* Theory:
*
* Side Effects:
*
* Note:
* None
*****************************************************************************/
static uint8_t TxModeSwitchHandler(char* out, TCmdTbl* tbl, char* args)
{
	UART_write(uart, "TxModeSwitchHandler", 19);

	switchRxTx = RFEASYLINKTX_MODE;
	is_mode_control = RFEASYLINKTX_MODE;

    return TRUE;
}


/*****************************************************************************
* Function Name: RxModeSwitchHandler()
******************************************************************************
* Summary:
*
* Parameters:
* None
*
* Return:
*
* Theory:
*
* Side Effects:
*
* Note:
* None
*****************************************************************************/
static uint8_t RxModeSwitchHandler(char* out, TCmdTbl* tbl, char* args)
{
	UART_write(uart, "RxModeSwitchHandler", 19);

	switchRxTx = RFEASYLINKRX_MODE;
	is_mode_control = RFEASYLINKRX_MODE;

    return TRUE;
}


/*****************************************************************************
* Function Name: GetIdResponseCommandHandler()
******************************************************************************
* Summary: transmitted from the wireless communication device to the module.
*
* Parameters:
* None
*
* Return:
*
* Theory:
*
* Side Effects:
*
* Note:
* None
*****************************************************************************/
static uint8_t GetIdResponseCommandHandler(char* out, TCmdTbl* tbl, char* args)
{
    uint8_t        i, k = 0, argC = 0;
    uint8_t        size = 0;
    unsigned char  h_hexval = 0, l_hexval = 0, hexval = 0;

    rxPacketDump.payload[size++] = 0x11;
    rxPacketDump.payload[size++] = 0x01;

    for(i=0; i < (_TPMS_SERIAL_NUMBER_LEN); i++)
    {
        k++;

        if(k % 2)
        {
            h_hexval  = (args[i] - '0')  << 4;
        }
        else
        {
            l_hexval  = (args[i] - '0');
            hexval = h_hexval | l_hexval;
            rxPacketDump.payload[size++] = hexval;
        }
        argC++;
    }

    for(i=0; i < (_TPMS_CAR_NUMBER); i++)
    {
        rxPacketDump.payload[size++] = args[argC++];
    }

    rxPacketDump.payload[size++] = args[argC++];
    TxCommandHandler(rxPacketDump.payload, size);

    return TRUE;
}


/*****************************************************************************
* Function Name: SetInfoResponseCommandHandler()
******************************************************************************
* Summary: transmitted from the wireless communication device to the module.
*
* Parameters:
* None
*
* Return:
*
* Theory:
*
* Side Effects:
*
* Note:
* None
*****************************************************************************/
static uint8_t SetInfoResponseCommandHandler(char* out, TCmdTbl* tbl, char* args)
{
    uint8_t        i, k = 0,argC = 0;
    uint8_t        size = 0;
    unsigned char  h_hexval = 0, l_hexval = 0, hexval = 0;

    rxPacketDump.payload[size++] = 0x11;
    rxPacketDump.payload[size++] = 0x02;

    for(i=0; i < (_TPMS_SERIAL_NUMBER_LEN); i++)
    {
        k++;

        if(k % 2)
        {
            h_hexval  = (args[i] - '0')  << 4;
        }
        else
        {
            l_hexval  = (args[i] - '0');
            hexval = h_hexval | l_hexval;
            rxPacketDump.payload[size++] = hexval;
        }
        argC++;
    }

    rxPacketDump.payload[size++] = args[argC++];

    rxPacketDump.payload[size++] = args[argC];
    TxCommandHandler(rxPacketDump.payload, size);

    return TRUE;
}


/*****************************************************************************
* Function Name: SetCompleteResponseCommandHandler()
******************************************************************************
* Summary: transmitted from the wireless communication device to the module.
*
* Parameters:
* None
*
* Return:
*
* Theory:
*
* Side Effects:
*
* Note:
* None
*****************************************************************************/
static uint8_t SetCompleteResponseCommandHandler(char* out, TCmdTbl* tbl, char* args)
{
    uint8_t        i, k = 0,argC = 0;
    uint8_t        size = 0;
    unsigned char  h_hexval = 0, l_hexval = 0, hexval = 0;

    rxPacketDump.payload[size++] = 0x11;
    rxPacketDump.payload[size++] = 0x03;

    for(i=0; i < (_TPMS_SERIAL_NUMBER_LEN); i++)
    {
        k++;

        if(k % 2)
        {
            h_hexval  = (args[i] - '0')  << 4;
        }
        else
        {
            l_hexval  = (args[i] - '0');
            hexval = h_hexval | l_hexval;
            rxPacketDump.payload[size++] = hexval;
        }
        argC++;
    }

    rxPacketDump.payload[size++] = args[argC++];
    TxCommandHandler(rxPacketDump.payload, size);

    return TRUE;
}


/*****************************************************************************
* Function Name: DeleteInfoResponseCommandHandler()
******************************************************************************
* Summary: transmitted from the wireless communication device to the module.
*
* Parameters:
* None
*
* Return:
*
* Theory:
*
* Side Effects:
*
* Note:
* None
*****************************************************************************/
static uint8_t DeleteInfoResponseCommandHandler(char* out, TCmdTbl* tbl, char* args)
{
    uint8_t        i, k = 0, argC = 0;
    uint8_t        size = 0;
    unsigned char  h_hexval = 0, l_hexval = 0, hexval = 0;

    rxPacketDump.payload[size++] = 0x11;
    rxPacketDump.payload[size++] = 0x04;

    for(i=0; i < (_TPMS_SERIAL_NUMBER_LEN); i++)
    {
        k++;

        if(k % 2)
        {
            h_hexval  = (args[i] - '0')  << 4;
        }
        else
        {
            l_hexval  = (args[i] - '0');
            hexval = h_hexval | l_hexval;
            rxPacketDump.payload[size++] = hexval;
        }
        argC++;
    }

    rxPacketDump.payload[size++] = args[argC++];
    TxCommandHandler(rxPacketDump.payload, size);

    return TRUE;
}


/*****************************************************************************
* Function Name: NewStockeResponseCommandHandler()
******************************************************************************
* Summary: transmitted from the wireless communication device to the module.
*
* Parameters:
* None
*
* Return:
*
* Theory:
*
* Side Effects:
*
* Note:
* None
*****************************************************************************/
static uint8_t NewStockeResponseCommandHandler(char* out, TCmdTbl* tbl, char* args)
{
    uint8_t        i, k = 0, argC = 0;
    uint8_t        size = 0;
    unsigned char  h_hexval = 0, l_hexval = 0, hexval = 0;

    rxPacketDump.payload[size++] = 0x12;
    rxPacketDump.payload[size++] = 0x01;

    for(i=0; i < (_TPMS_SERIAL_NUMBER_LEN); i++)
    {
        k++;

        if(k % 2)
        {
            h_hexval  = (args[i] - '0')  << 4;
        }
        else
        {
            l_hexval  = (args[i] - '0');
            hexval = h_hexval | l_hexval;
            rxPacketDump.payload[size++] = hexval;
        }
        argC++;
    }

    rxPacketDump.payload[size++] = args[argC];
    TxCommandHandler(rxPacketDump.payload, size);

    return TRUE;
}


/*****************************************************************************
* Function Name: RestockeResponseCommandHandler()
******************************************************************************
* Summary: transmitted from the wireless communication device to the module.
*
* Parameters:
* None
*
* Return:
*
* Theory:
*
* Side Effects:
*
* Note:
* None
*****************************************************************************/
static uint8_t RestockeResponseCommandHandler(char* out, TCmdTbl* tbl, char* args)
{
    uint8_t        i, k = 0, argC = 0;
    uint8_t        size = 0;
    unsigned char  h_hexval = 0, l_hexval = 0, hexval = 0;

    rxPacketDump.payload[size++] = 0x12;
    rxPacketDump.payload[size++] = 0x02;

    for(i=0; i < (_TPMS_SERIAL_NUMBER_LEN); i++)
    {
        k++;

        if(k % 2)
        {
            h_hexval  = (args[i] - '0')  << 4;
        }
        else
        {
            l_hexval  = (args[i] - '0');
            hexval = h_hexval | l_hexval;
            rxPacketDump.payload[size++] = hexval;
        }
        argC++;
    }

    rxPacketDump.payload[size++] = args[argC];
    TxCommandHandler(rxPacketDump.payload, size);

    return TRUE;
}


/*****************************************************************************
* Function Name: DisposeResponseCommandHandler()
******************************************************************************
* Summary: transmitted from the wireless communication device to the module.
*
* Parameters:
* None
*
* Return:
*
* Theory:
*
* Side Effects:
*
* Note:
* None
*****************************************************************************/
static uint8_t DisposeResponseCommandHandler(char* out, TCmdTbl* tbl, char* args)
{
    uint8_t        i, k = 0, argC = 0;
    uint8_t        size = 0;
    unsigned char  h_hexval = 0, l_hexval = 0, hexval = 0;

    rxPacketDump.payload[size++] = 0x12;
    rxPacketDump.payload[size++] = 0x03;

    for(i=0; i < (_TPMS_SERIAL_NUMBER_LEN); i++)
    {
        k++;

        if(k % 2)
        {
            h_hexval  = (args[i] - '0')  << 4;
        }
        else
        {
            l_hexval  = (args[i] - '0');
            hexval = h_hexval | l_hexval;
            rxPacketDump.payload[size++] = hexval;
        }
        argC++;
    }

    rxPacketDump.payload[size++] = args[argC];
    TxCommandHandler(rxPacketDump.payload, size);

    return TRUE;
}

#ifdef FEATURE_SLEEP_MODE
/*****************************************************************************
* Function Name: SleepModeResponseCommandHandler()
******************************************************************************
* Summary: transmitted from the wireless communication device to the module.
*
* Parameters:
* None
*
* Return:
*
* Theory:
*
* Side Effects:
*
* Note:
* None
*****************************************************************************/
static uint8_t SleepModeResponseCommandHandler(char* out, TCmdTbl* tbl, char* args)
{
	switchRxTx = RFEASYLINK_NONE_MODE;
	is_mode_control = RFEASYLINK_NONE_MODE;

	GPIO_write(CONFIG_GPIO_RF_LNA_ENABLE, 0);
	GPIO_write(CONFIG_GPIO_RF_SWITCH1, 0);
	GPIO_write(CONFIG_GPIO_RF_SWITCH2, 0);
	GPIO_write(CONFIG_GPIO_RTS, 0);

    UART_close(uart);

	rfEasyLinkPowerManagement();
    return TRUE;
}
#endif


/***** Function definitions *****/
#ifdef RFEASYLINKRX_ASYNC
void rxDoneCb(EasyLink_RxPacket * rxPacket, EasyLink_Status status)
{
    if (status == EasyLink_Status_Success)
    {
        /* Toggle RLED to indicate RX */
        PIN_setOutputValue(pinHandle, CONFIG_PIN_RLED,!PIN_getOutputValue(CONFIG_PIN_RLED));
    }
    else if(status == EasyLink_Status_Aborted)
    {
        /* Toggle GLED to indicate command aborted */
        PIN_setOutputValue(pinHandle, CONFIG_PIN_GLED,!PIN_getOutputValue(CONFIG_PIN_GLED));
    }
    else
    {
        /* Toggle GLED and RLED to indicate error */
        PIN_setOutputValue(pinHandle, CONFIG_PIN_GLED,!PIN_getOutputValue(CONFIG_PIN_GLED));
        PIN_setOutputValue(pinHandle, CONFIG_PIN_RLED,!PIN_getOutputValue(CONFIG_PIN_RLED));
    }

    Semaphore_post(rxDoneSem);
}
#endif

static void rfEasyLinkRxFnx(UArg arg0, UArg arg1)
{
    uint32_t absTime;

#ifndef RFEASYLINKRX_ASYNC
    EasyLink_RxPacket rxPacket = {0};
#endif

#ifdef RFEASYLINKRX_ASYNC
    /* Create a semaphore for Async*/
    Semaphore_Params params;
    Error_Block eb;

    /* Init params */
    Semaphore_Params_init(&params);
    Error_init(&eb);

    /* Create semaphore instance */
    rxDoneSem = Semaphore_create(0, &params, &eb);
    if(rxDoneSem == NULL)
    {
        System_abort("Semaphore creation failed");
    }

#endif //RFEASYLINKRX_ASYNC

    // Initialize the EasyLink parameters to their default values
    EasyLink_Params_init(&easyLink_params);

    /*
     * Initialize EasyLink with the settings found in ti_easylink_config.h
     * Modify EASYLINK_PARAM_CONFIG in ti_easylink_config.h to change the default
     * PHY
     */
    if(EasyLink_init(&easyLink_params) != EasyLink_Status_Success)
    {
        System_abort("EasyLink_init failed");
    }

    /*
     * If you wish to use a frequency other than the default, use
     * the following API:
     * EasyLink_setFrequency(868000000);
     */

    while(1) {
#ifdef RFEASYLINKRX_ASYNC
        EasyLink_receiveAsync(rxDoneCb, 0);

        /* Wait 300ms for Rx */
        if(Semaphore_pend(rxDoneSem, (300000 / Clock_tickPeriod)) == FALSE)
        {
            /* RX timed out abort */
            if(EasyLink_abort() == EasyLink_Status_Success)
            {
               /* Wait for the abort */
               Semaphore_pend(rxDoneSem, BIOS_WAIT_FOREVER);
            }
        }
#else
		if(switchRxTx == RFEASYLINKRX_MODE)
		{
            GPIO_write(CONFIG_GPIO_RF_LNA_ENABLE, 1);
            GPIO_write(CONFIG_GPIO_RF_SWITCH1, 1);
            GPIO_write(CONFIG_GPIO_RF_SWITCH2, 0);

            EasyLink_Status result = EasyLink_receive(&rxPacket);

            if (result == EasyLink_Status_Success) {
                rxPacketDump.len = rxPacket.len;
                rxPacketDump.rssi = rxPacket.rssi;
                memcpy(&rxPacketDump.payload, &rxPacket.payload, EASYLINK_MAX_DATA_LENGTH);
                rfRxCommandData();
            }
		}
		else if(switchRxTx == RFEASYLINKTX_MODE)
		{
            //int i;
			GPIO_write(CONFIG_GPIO_RF_LNA_ENABLE, 0);
			GPIO_write(CONFIG_GPIO_RF_SWITCH1, 0);
			GPIO_write(CONFIG_GPIO_RF_SWITCH2, 1);
#if 0 // for rf tx testing
            if(EasyLink_getAbsTime(&absTime) != EasyLink_Status_Success)
			{
				// Problem getting absolute time
			}
			txPacket.absTime = absTime + EasyLink_ms_To_RadioTime(1000);

        	//EasyLink_Status result = EasyLink_transmit(&txPacket);
			txPacket.payload[0] = (uint8_t)(seqNumber >> 8);
			txPacket.payload[1] = (uint8_t)(seqNumber++);
			for (i = 2; i < RFEASYLINKECHO_PAYLOAD_LENGTH; i++)
			{
				txPacket.payload[i] = rand();
			}

			txPacket.len = RFEASYLINKECHO_PAYLOAD_LENGTH;

			/*
			 * Address filtering is enabled by default on the Rx device with the
			 * an address of 0xAA. This device must set the dstAddr accordingly.
			 */
			txPacket.dstAddr[0] = 0xaa;

			/* Set Tx absolute time to current time + 1000ms */
			if(EasyLink_getAbsTime(&absTime) != EasyLink_Status_Success)
			{
				// Problem getting absolute time
			}
			txPacket.absTime = absTime + EasyLink_ms_To_RadioTime(100);

        	EasyLink_Status result = EasyLink_transmit(&txPacket);
#endif
			txPacket.dstAddr[0] = 0xaa;

			/* Set Tx absolute time to current time + 1000ms */
			if(EasyLink_getAbsTime(&absTime) != EasyLink_Status_Success)
			{
				// Problem getting absolute time
			}
			txPacket.absTime = absTime + EasyLink_ms_To_RadioTime(100);

			if(g_tx_trigger) {
			    EasyLink_Status result = EasyLink_transmit(&txPacket);
			    if(result == EasyLink_Status_Success) {
			        g_tx_trigger = 0;
			        switchRxTx = RFEASYLINKRX_MODE;
                    SysCtrlSystemReset();
			    }
			}
		}
#ifdef FEATURE_SLEEP_MODE
        else if(switchRxTx == RFEASYLINK_NONE_MODE)
        {
           break;
        }
#endif
#endif //RX_ASYNC
    }
}


/*****************************************************************************
* Function Name: uartTaskFunction()
******************************************************************************
* Summary:
*
* Parameters:
* None
*
* Return:
*
* Theory:
*
* Side Effects:
*
* Note:
* None
*****************************************************************************/
static void uartTaskFunction(UArg arg0, UArg arg1)
{
    uint16_t         msgLength = 0;
    uint8_t          input;
	uint32_t         status;

	while (1)
    {
		msgLength = 0;

		while (1) {

#ifdef FEATURE_SLEEP_MODE
            if(switchRxTx == RFEASYLINK_NONE_MODE)
            {
               break;
            }
#endif
			status = UART_read(uart, &input, 1);

			if(status == 0)
				break;

			if (input != 0x0D) {
			}
			else {
                rfEasyLink_Uart_init();
                rfEasyLinkInit();

				ParserReceiverMgr(msg, msgLength);
				memset(msg, 0x00, sizeof(msg));
				break;
			}

			msg[msgLength] = input;
			msgLength++;

			if (msgLength == MAX_MSG_LENGTH) {
				memset(msg, 0x00, sizeof(msg));
				break;
			}
		}
#ifdef FEATURE_SLEEP_MODE
		if(switchRxTx == RFEASYLINK_NONE_MODE)
		{
			break;
		}
#endif
    }
}


/*****************************************************************************
* Function Name: StrDiff()
******************************************************************************
* Summary:
*
* Parameters:
* None
*
* Return:
*
* Theory:
*
* Side Effects:
*
* Note:
* None
*****************************************************************************/
static uint8_t StrDiff
(
  char* a,
  char* b,
  uint32_t len
)
{
   int i;

   for( i = 0; i < len; i++ )
   {
      if(IS_SMALL_LETTER(*a)){
         *a=*a-0x20;  /* change to Capital letter */
      }
      if( *a++ != *b++ ) return FALSE;
   }

   return TRUE;
}


/*=========================================================================
  FUNCTION:  ParserReceiverMgr
===========================================================================*/
/*!
    @brief

    @return

*/
/*=========================================================================*/
void ParserReceiverMgr(uint8_t *mMsg, int len)
{
    int i;
    uint8_t ret = 0;

    for( i = 0; i < ARR_SIZE(parserCmds); i++)
    {
        if(StrDiff((char*)mMsg, parserCmds[i].cmd, parserCmds[i].len))
        {
            if(parserCmds[i].hdl)
            {
                ret = parserCmds[i].hdl((char*)mMsg, (TCmdTbl*)&parserCmds[i], (char*)(mMsg+parserCmds[i].len));
            }

            if(ret) break;
        }
    }
}

/*=========================================================================
  FUNCTION:  rfEasyLink_Uart_init
===========================================================================*/
/*!
    @brief

    @return

*/
/*=========================================================================*/
void rfEasyLink_Uart_init(void)
{
    UART_close(uart);
    UART_init();

    // Create a UART with data processing off.
    UART_Params_init(&uartParams);
    uartParams.writeDataMode = UART_DATA_BINARY;
    uartParams.readDataMode = UART_DATA_BINARY;
    uartParams.readReturnMode = UART_RETURN_FULL;
    //uartParams.readTimeout = (10000 / Clock_tickPeriod); //return after blocking for 10ms
    uartParams.readEcho = UART_ECHO_OFF;
    uartParams.baudRate = 115200;
    uart = UART_open(CONFIG_UART_0, &uartParams);
}


void rxTask_init(PIN_Handle ledPinHandle) {

    pinHandle = ledPinHandle;

    Task_Params_init(&rxTaskParams);
    rxTaskParams.stackSize = RFEASYLINKRX_TASK_STACK_SIZE;
    rxTaskParams.priority = RFEASYLINKRX_TASK_PRIORITY;
    rxTaskParams.stack = &rxTaskStack;
    //rxTaskParams.arg0 = (UInt)1000000;

    Task_construct(&rxTask, rfEasyLinkRxFnx, &rxTaskParams, NULL);

    rxTaskParams.priority = UART_TASK_PRIORITY;
    rxTaskParams.stack = &uartTaskStack;
    //rxTaskParams.arg0 = (UInt)1000000;

    Task_construct(&uartTask, uartTaskFunction, &rxTaskParams, NULL);
}


/*
 *  ======== main ========
 */
int main(void)
{
    /* Call driver init functions */
    Board_initGeneral();
    /* Open LED pins */
//    ledPinHandle = PIN_open(&ledPinState, pinTable);
//    Assert_isTrue(ledPinHandle != NULL, NULL);
    GPIO_init();

    /* Clear LED pins */
//    PIN_setOutputValue(ledPinHandle, CONFIG_PIN_GLED, 0);
//    PIN_setOutputValue(ledPinHandle, CONFIG_PIN_RLED, 0);

    // Initialize the UART driver.  UART_init() must be called before
    // calling any other UART APIs.
    UART_init();

    // Create a UART with data processing off.
    UART_Params_init(&uartParams);
    uartParams.writeDataMode = UART_DATA_BINARY;
    uartParams.readDataMode = UART_DATA_BINARY;
    uartParams.readReturnMode = UART_RETURN_FULL;
    //uartParams.readTimeout = (10000 / Clock_tickPeriod); //return after blocking for 10ms
    uartParams.readEcho = UART_ECHO_OFF;
    uartParams.baudRate = 115200;
    uart = UART_open(CONFIG_UART_0, &uartParams);

    rxTask_init(ledPinHandle);

    /* Start BIOS */
    BIOS_start();
    return (0);
}


